/* $Id$ */

/*
 * This file is part of OpenTTD.
 * OpenTTD is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 2.
 * OpenTTD is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details. You should have received a copy of the GNU General Public License along with OpenTTD. If not, see <http://www.gnu.org/licenses/>.
 */

#include "../stdafx.h"
#include "../mixer.h"
#include "../core/alloc_func.hpp"
#include "aesnd_s.h"
#include <aesndlib.h>

static FSoundDriver_AESND iFSoundDriver_AESND;

static AESNDPB *voice;
static byte buffer[DSP_STREAMBUFFER_SIZE * 2];

static void VoiceStreamCallback(AESNDPB *pb, uint32 state)
{
	if (state == VOICE_STATE_STREAM) {
		MxMixSamples(buffer, sizeof(buffer) / 4);
		AESND_SetVoiceBuffer(pb, buffer, sizeof(buffer));
	}
}

const char *SoundDriver_AESND::Start(const char * const *parm)
{
	AESND_Init();

	voice = AESND_AllocateVoice(VoiceStreamCallback);
	if (voice == NULL) return "Failed to allocate voice";

	int hz = GetDriverParamInt(parm, "hz", 44100);
	MxInitialize(hz);

	AESND_SetVoiceFormat(voice, VOICE_STEREO16);
	AESND_SetVoiceFrequency(voice, hz);
	AESND_SetVoiceStream(voice, true);
	AESND_SetVoiceStop(voice, false);

	return NULL;
}

void SoundDriver_AESND::Stop()
{
	AESND_FreeVoice(voice);
	voice = NULL;
}
